import java.util.ArrayList;
import java.util.HashSet;
import java.util.Stack;

/**
 * SolvabilitySearcher.java - a simple recursive implementation of
 * depth-first search with cached solvable/unsolvable state Strings.
 *
 * @author Todd Neller
 * @version 1.0
 */

public class SolvabilitySearcher extends Searcher {

	HashSet<String> solvable = new HashSet<String>();
	HashSet<String> unsolvable = new HashSet<String>();

	/**
	 * <code>search</code> - given an initial node, perform recursive
	 * depth-first search (DFS), caching (un)solvable node Strings
	 *
	 * @param node a <code>SearchNode</code> value - the node to be searched
	 * @return a <code>boolean</code> value - whether or not goal node
	 * was found */
	public boolean search(SearchNode node) {
		nodeCount++;
		String nodeString = node.toString();
		if (unsolvable.contains(nodeString))
			return false;
		if (solvable.contains(nodeString))
			return true;
		if (node.isGoal()) {
			goalNode = node;
			solvable.add(nodeString);
			return true;
		}
		else {
			ArrayList<SearchNode> children = node.expand();
			for (SearchNode child : children)
				if (search(child)) {
					solvable.add(nodeString);
					return true;
				}
			unsolvable.add(nodeString);
			return false;
		}
	}
}


